#> iris_rempot:raycast/check_intersection/ray_plane
#
# Tests for ray-plane intersections with a face
#
# @reads
#   storage iris_rempot:data Face: compound
#       Direction: string
#           The axis that is perpendicular to this face
#           One of WEST_EAST, UP_DOWN, NORTH_SOUTH
#       min: float[]
#       max: float[]
# @output
#   Success: 1 if the face is hit, 0 otherwise
#   Result: the distance, in mm, before the face is hit
# @private
# @within iris_rempot:raycast/check_intersection/ray_box

# Save face coordinates
execute store result score $x0 iris_rempot run data get storage iris_rempot:data Face.min[0] 1000000
execute store result score $y0 iris_rempot run data get storage iris_rempot:data Face.min[1] 1000000
execute store result score $z0 iris_rempot run data get storage iris_rempot:data Face.min[2] 1000000
execute store result score $x1 iris_rempot run data get storage iris_rempot:data Face.max[0] 1000000
execute store result score $y1 iris_rempot run data get storage iris_rempot:data Face.max[1] 1000000
execute store result score $z1 iris_rempot run data get storage iris_rempot:data Face.max[2] 1000000

# Get distance (in mm) to the plane, i.e. how long the ray should extend before it hits the plane
# This value should be at most sqrt(3)*1000; if it's more than 2000, we fail (otherwise we risk overflow errors)
execute if data storage iris_rempot:data {Face: {Direction: "WEST_EAST"}} run scoreboard players operation $distance iris_rempot = $x0 iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "WEST_EAST"}} run scoreboard players operation $distance iris_rempot -= ${x} iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "UP_DOWN"}} run scoreboard players operation $distance iris_rempot = $y0 iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "UP_DOWN"}} run scoreboard players operation $distance iris_rempot -= ${y} iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "NORTH_SOUTH"}} run scoreboard players operation $distance iris_rempot = $z0 iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "NORTH_SOUTH"}} run scoreboard players operation $distance iris_rempot -= ${z} iris_rempot
scoreboard players operation $distance iris_rempot *= $1000 iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "WEST_EAST"}} run scoreboard players operation $distance iris_rempot /= $dx iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "UP_DOWN"}} run scoreboard players operation $distance iris_rempot /= $dy iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "NORTH_SOUTH"}} run scoreboard players operation $distance iris_rempot /= $dz iris_rempot
execute if score $distance iris_rempot matches ..-1 run return fail
execute if score $distance iris_rempot matches 2000.. run return fail

# Get x position of the ray/plane intersection
scoreboard players operation $x_intersection iris_rempot = $distance iris_rempot
scoreboard players operation $x_intersection iris_rempot *= $dx iris_rempot
scoreboard players operation $x_intersection iris_rempot /= $1000 iris_rempot
scoreboard players operation $x_intersection iris_rempot += ${x} iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "WEST_EAST"}} run scoreboard players operation $x_intersection iris_rempot = $x0 iris_rempot

# Get y position of the ray/plane intersection
scoreboard players operation $y_intersection iris_rempot = $distance iris_rempot
scoreboard players operation $y_intersection iris_rempot *= $dy iris_rempot
scoreboard players operation $y_intersection iris_rempot /= $1000 iris_rempot
scoreboard players operation $y_intersection iris_rempot += ${y} iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "UP_DOWN"}} run scoreboard players operation $y_intersection iris_rempot = $y0 iris_rempot

# Get z position of the ray/plane intersection
scoreboard players operation $z_intersection iris_rempot = $distance iris_rempot
scoreboard players operation $z_intersection iris_rempot *= $dz iris_rempot
scoreboard players operation $z_intersection iris_rempot /= $1000 iris_rempot
scoreboard players operation $z_intersection iris_rempot += ${z} iris_rempot
execute if data storage iris_rempot:data {Face: {Direction: "NORTH_SOUTH"}} run scoreboard players operation $z_intersection iris_rempot = $z0 iris_rempot

# Save the position of the intersection
data modify storage iris_rempot:data Face.Collision set value [0.0d, 0.0d, 0.0d]
execute store result storage iris_rempot:data Face.Collision[0] double 0.000001 run scoreboard players get $x_intersection iris_rempot
execute store result storage iris_rempot:data Face.Collision[1] double 0.000001 run scoreboard players get $y_intersection iris_rempot
execute store result storage iris_rempot:data Face.Collision[2] double 0.000001 run scoreboard players get $z_intersection iris_rempot

# See if that intersection falls within the face
execute if score $x_intersection iris_rempot >= $x0 iris_rempot if score $x_intersection iris_rempot <= $x1 iris_rempot \
        if score $y_intersection iris_rempot >= $y0 iris_rempot if score $y_intersection iris_rempot <= $y1 iris_rempot \
        if score $z_intersection iris_rempot >= $z0 iris_rempot if score $z_intersection iris_rempot <= $z1 iris_rempot \
        run return run scoreboard players get $distance iris_rempot

return fail
